import setuptools



setuptools.setup(
    name="Backtester_Tushar",
    version="0.4.9",
    author="Tushar Sarkar",
    author_email="tusharsarkar866@gmail.com",
    description="Backtesting Package",
    long_description="Backtesting Framework for Tushar",
    long_description_content_type="text/markdown",
    url="https://github.com/tusharsarkar3/Backtester_Tushar",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
